from urllib.parse import urlparse
import boto3
from pystac import STAC_IO

from .. import add, IoReadWrite


def s3_read_text_method(uri):
    parsed = urlparse(uri)
    if parsed.scheme == "s3":
        bucket = parsed.netloc
        key = parsed.path.lstrip("/")
        s3 = boto3.resource("s3")
        obj = s3.Object(bucket, key)
        return obj.get()["Body"].read().decode("utf-8")
    else:
        return STAC_IO.default_read_text_method(uri)


def s3_write_text_method(uri, txt):
    parsed = urlparse(uri)
    if parsed.scheme == "s3":
        bucket = parsed.netloc
        key = parsed.path.lstrip("/")
        s3 = boto3.resource("s3")
        s3.Object(bucket, key).put(Body=txt)
    else:
        STAC_IO.default_write_text_method(uri, txt)


add("s3", IoReadWrite(s3_read_text_method, s3_write_text_method))



from urllib.parse import urlparse
import requests
from pystac import STAC_IO

from .. import add, IoReadWrite


def https_read_text_method(uri):
    parsed = urlparse(uri)
    if parsed.scheme.startswith("http"):
        return requests.get(uri).text
    else:
        return STAC_IO.default_read_text_method(uri)


add("https", IoReadWrite(https_read_text_method, None))



from collections import namedtuple
from contextlib import AbstractContextManager
from typing import Dict, List

from pystac import STAC_IO

"""Container for PSTAC_IO read write methods

read: pystac.STAC_IO.read_text_method or None
write: pystac.STAC_IO.write_text_method or None

"""
IoReadWrite = namedtuple("IoReadWrite", ["read", "write"])


""" Private dict of registered IO modules """
PYSTAC_IO: Dict[str, IoReadWrite] = {}


def add(module_name, io_read_write):
    """Add a new pystac_io module

    This method should be called by any module that implements new read and write IO methods
    during init so that they're added to the list of avaiable IO modules.

    See pystac_io.https for an example.

    """
    if module_name in PYSTAC_IO:
        raise ValueError("pystac_io.{} already loaded!".format(module_name))
    PYSTAC_IO.update({module_name: io_read_write})


def loaded_modules() -> List[str]:
    """ Return a list of available pystac_io modules that can be enabled with register() """
    return PYSTAC_IO.keys()


class PyStacIoContextManager(AbstractContextManager):
    """ Manages loading and unloading of custom IO methods for pystac.STAC_IO """

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_value, traceback):
        unregister()
        return None

    def __call__(self, module_name):
        """Register module_name and return a context manager object

        Perform register here rather than in __enter__ so that we register for both
        context manager and regular function invocations

        """
        if module_name in PYSTAC_IO.keys():
            io = PYSTAC_IO[module_name]
            if io.read is not None:
                STAC_IO.read_text_method = io.read
            if io.write is not None:
                STAC_IO.write_text_method = io.write
        else:
            err_str = "pystac_io.register(module_name) must be one of {}".format(
                PYSTAC_IO.keys()
            )
            raise ValueError(err_str)
        return self


""" Replace pystac.STAC_IO handlers with handlers provided by pystac_io.{module_name} """
register = PyStacIoContextManager()


def unregister():
    """ Reset any registered pystac_io handlers to the default pystac.STAC_IO handlers """
    STAC_IO.read_text_method = STAC_IO.default_read_text_method
    STAC_IO.write_text_method = STAC_IO.default_write_text_method



from setuptools import find_packages, setup

setup(
    name="pystac-io",
    version="0.1.0",
    description="Python library providing additional IO implementations for PySTAC",
    author="Azavea",
    author_email="info@azavea.com",
    url="https://github.com/azavea/pystac-io",
    license="Apache Software License 2.0",
    packages=find_packages(),
    install_requires=["pystac>=0.5.2"],
    extras_require={"https": ["requests>=2.24.0"], "s3": ["boto3>=1.16.6"]},
    keywords=["pystac", "pystac_io", "pystac-io", "s3", "catalog", "STAC"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)
